require 'mechanize'

class Scraper
	URL = "https://socrates.devbootcamp.com/login"

	def initialize(email, password)
		@email = email
		@password = password

		@agent = Mechanize.new
	end

	def get_user_profiles
		cohort_page = get_cohort_page
		user_links = get_user_links(cohort_page)

		raise "Invalid username or password" if user_links.empty?

		puts "Getting user profiles..."
		create_user_profiles(user_links)
	end

	private

	def get_cohort_page
		@agent.get(URL)
		login
		@agent.get("https://socrates.devbootcamp.com/cohorts/78")
	end

	def login
		form = @agent.page.forms.first
		form.email = @email
		form.password = @password
		form.submit
	end

	def get_user_links(page)
		users = page.links_with(href: /users/)

		# remove top link 'My Profile'
		users.shift

		# deduplicate links (photo link + text link)
		users.uniq { |link| link.href }
	end

	def create_user_profiles(user_links)
		user_links.map do |link|
			# reset to cohort page after every user
			@agent.transact do
				user_html = extract_profile_html(link)
				convert_html_to_person_hash(user_html)
			end
		end
	end

	def extract_profile_html(link)
		page = @agent.click(link)

		# get the socrates id and convert into an
		# easily identifiable html element
		socrates_id = link.href[/.*\/(\d*)/, 1]
		socrates_tag = "<p class='soc_id'>#{socrates_id}</p>"

		Nokogiri::HTML(page.search('div.profile').to_s << socrates_tag)
	end

	def convert_html_to_person_hash(html)
		person = {}
		person[:name] = get_name(html)
		person[:image] = get_image(html)
		person[:facebook_id] = get_facebook_id(html)
		person
	end

	def get_name(html)
		html.css('h1 > text()').first.text.lstrip.rstrip
	end

	def get_image(html)
		html.css('.user > img').first['src']
	end

	def get_facebook_id(html)
		html.css('dd')[4].text[/.*\/(.*)/, 1]
	end
end
